SystemOrganization addCategory: #'Volunti-MPDVS-Model'!
SystemOrganization addCategory: #'Volunti-MPDVS-View'!
SystemOrganization addCategory: #'Volunti-MPDVS-Tests'!

TestCase subclass: #VOMPDVSTest
	instanceVariableNames: 'mpdvs privateKey polynome gen'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Tests'!

!VOMPDVSTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 7/23/2012 07:45'!
setUp
	mpdvs := VOMPDVSAuthority new.
	gen := mpdvs setup: CryptographyHelper rsaKeyLength! !

!VOMPDVSTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 3/28/2012 10:24'!
testComplexMPDVS
	"Complete process for 3 participants"

"	| generator a1 a2 shares function d di a1Correction share prod invDen m invCorr partialSign signature sk correctSign |
	shares := Array new: 3.
	d := OrderedCollection new.
	generator := VORSAKeyPairGenerator bits: 1024.
	a1 := Array with: 1 with: 3.
	a2 := Array with: 2 with: 3.
	function := VOPolynome order: 1 constant: generator d max: generator n.
	1 to: shares size do: [ :i | shares at: i put: (function of: i mod: generator n) ].
	self halt.
	a1
		do: [ :i | 
			share := shares at: i.
			prod := 1.
			a1
				do: [ :j | 
					i = j
						ifFalse: [ prod := prod * (j / (j - i)) ] ].
			di := share * prod.
			di isFraction
				ifTrue: [ 
					invDen := (ExtendedEuclidAlgorithm with: di denominator with: generator n) inverseModulo.
					di := di numerator * invDen ].
			d add: di \\ generator n ].
	a1Correction := (d sum - generator d) / generator n.
	m := 6.
	invCorr := (ExtendedEuclidAlgorithm with: (m raisedTo: a1Correction * generator n) with: generator n) inverseModulo.
	partialSign := 1.
	d do: [ :dj | partialSign := partialSign * (m raisedTo: dj modulo: generator n) ].
	signature := partialSign * invCorr \\ generator n.
	sk := generator privateKey.
	correctSign := sk crypt: m.
	self assert: signature greaseString = correctSign greaseString"! !

!VOMPDVSTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 3/28/2012 15:13'!
testCreateShares
	| secrets m mbSecret userSecret witnessesPartialSign partialSign signature sk sign |
	secrets := mpdvs keyGen.
	sk := gen privateKey.
	m := 6.
	mbSecret := secrets first.
	userSecret := secrets second.
	witnessesPartialSign := 1.
	3 to: secrets size do: [ :i | witnessesPartialSign := witnessesPartialSign * ((secrets at: i) sign: m) ].
	partialSign := (mbSecret sign: m) * witnessesPartialSign \\ gen n.
	signature := partialSign * (mbSecret correctionFor: m) \\ gen n.
	sign := sk crypt: m.
	self assert: signature = sign! !

!VOMPDVSTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 5/9/2012 10:09'!
testSimpleMPDVS
	"Simplify complete process for 3 participants"

	| generator a1 a2 shares function d di a1Correction share prod invDen m invCorr partialSign signature |
	shares := Array new: 3.
	d := OrderedCollection new.
	generator := VORSAKeyPairGenerator new.
	generator p: 5.
	generator q: 7.
	generator e: 19.
	a1 := Array with: 1 with: 3.
	a2 := Array with: 2 with: 3.
	self assert: generator d = 19.
	function := VOPolynome order: 1 constant: generator d coefficients: (Array with: 6).
	1 to: shares size do: [ :i | shares at: i put: (function of: i mod: generator n) ].
	self assert: shares first = 25.
	self assert: shares second = 31.
	self assert: shares third = 2.
	d := a1
		collect: [ :i | 
			share := shares at: i.
			prod := 1.
			a1
				do: [ :j | 
					i = j
						ifFalse: [ prod := prod * (j / (j - i)) ] ].
			di := share * prod.
			di isFraction
				ifTrue: [ 
					invDen := (ExtendedEuclidAlgorithm with: di denominator with: generator n) inverseModulo.
					di := di numerator * invDen ].
			di \\ generator n ].
	self assert: d first = 20.
	self assert: d second = 34.
	a1Correction := (d sum - generator d) / generator n.
	self assert: a1Correction = 1.
	m := 6.
	invCorr := (ExtendedEuclidAlgorithm with: (m raisedTo: a1Correction * generator n) with: generator n) inverseModulo.
	self assert: invCorr = 6.
	partialSign := 1.
	d do: [ :dj | partialSign := partialSign * (m raisedTo: dj modulo: generator n) ].
	self assert: partialSign = 1.
	signature := partialSign * invCorr \\ generator n.
	self assert: signature = 6! !

TestCase subclass: #VOPolynomeTest
	instanceVariableNames: 'order constant coefficients polynome'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Tests'!

!VOPolynomeTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/5/2012 14:47'!
constant
	^ 5! !

!VOPolynomeTest methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/5/2012 14:47'!
order
	^ 3! !

!VOPolynomeTest methodsFor: 'running' stamp: 'NicolasVanderAuwera 3/10/2012 15:56'!
setUp
	"f(x) := 5 + 2 x + 3 x^2 + 4 x^3"

	order := self order.
	constant := self constant.
	polynome := VOPolynome order: order constant: constant coefficients: (Array with: 2 with: 3 with: 4)! !

!VOPolynomeTest methodsFor: 'tests' stamp: 'NicolasVanderAuwera 1/13/2012 10:37'!
testOf
	self assert: (polynome of: 5) = 590
	! !

VOAgentComponent subclass: #VOMPDVSComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-View'!

!VOMPDVSComponent class methodsFor: 'class initialization' stamp: 'NicolasVanderAuwera 5/5/2012 07:27'!
initialize
	super initialize.
	self register! !

!VOMPDVSComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/4/2012 15:57'!
url
	^ 'mpdvs'! !

!VOMPDVSComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:39'!
agentClass
	^ VOMPDVSManager! !

!VOMPDVSComponent methodsFor: 'updating' stamp: 'NicolasVanderAuwera 10/29/2012 14:41'!
main
	^ VOMPDVSMainAreaComponent! !

!VOMPDVSComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/5/2012 09:18'!
title
	^ 'MPDVS Authority'! !

VOAgentComponent subclass: #VOWitnessAuthorityComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-View'!

!VOWitnessAuthorityComponent class methodsFor: 'class initialization' stamp: 'NicolasVanderAuwera 5/5/2012 14:28'!
initialize
	super initialize.
	self register! !

!VOWitnessAuthorityComponent class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 07:51'!
url
	^ 'witnesses'! !

!VOWitnessAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:39'!
agentClass
	^ VOWitnessManager! !

!VOWitnessAuthorityComponent methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 14:11'!
title
	^ 'Witness Authority'! !

VOMainAreaComponent subclass: #VOMPDVSMainAreaComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-View'!

!VOMPDVSMainAreaComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 5/5/2012 12:11'!
renderConfigureAnchorText: text On: html
	html anchor
		callback: [ self call: '' ];
		with: text! !

!VOMPDVSMainAreaComponent methodsFor: 'rendering' stamp: 'NicolasVanderAuwera 4/11/2012 06:32'!
renderRestrictedContentOn: html
	html paragraph
		class: 'info';
		with: 'Witnesses are correctly configured.'! !

VOAuthority subclass: #VOMPDVSAuthority
	instanceVariableNames: 'numberOfParticipants groups generator shares'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOMPDVSAuthority commentStamp: 'NicolasVanderAuwera 5/14/2012 12:40' prior: 0!
I split a RSA private key in shares for the mutli-party designated verifier signature scheme, depending of the number of participants.

Instance Variables:
	numberOfParticipants	<ProtoObject | PseudoContext> voter + mental booth + number of witnesses.
	groups	<(Collection of: (ProtoObject | PseudoContext))> contains the two groups of participants.
	generator	<RSAKeyPairGenerator> used to generate the public and the private keys.
	shares	<Object> stores created shares.!

!VOMPDVSAuthority class methodsFor: 'generator' stamp: 'NicolasVanderAuwera 3/28/2012 08:26'!
generate: size sharesWith: aConstant modulo: n
	| polynome |
	polynome := VOPolynome order: size constant: aConstant max: n.
	^ (1 to: size) collect: [ :i | polynome of: i mod: n ]! !

!VOMPDVSAuthority class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 3/28/2012 09:35'!
numberOfParticipants: anInteger
	^ self new numberOfParticipants: anInteger! !

!VOMPDVSAuthority methodsFor: 'cryptography-mpdvs' stamp: 'NicolasVanderAuwera 3/28/2012 10:05'!
correctionFor: group
	^ (group sum - generator d) / generator n! !

!VOMPDVSAuthority methodsFor: 'cryptography-mpdvs' stamp: 'NicolasVanderAuwera 3/28/2012 09:48'!
groups
	groups
		ifNil: [ groups := Array with: #(1) , (3 to: self numberOfParticipants) with: (2 to: self numberOfParticipants) asArray ].
	^ groups! !

!VOMPDVSAuthority methodsFor: 'cryptography-mpdvs' stamp: 'NicolasVanderAuwera 3/28/2012 10:42'!
keyGen
	| secrets corrections verifierSecrets witnessSecrets |
	secrets := self groups collect: [ :group | self secretsFor: group ].
	corrections := secrets collect: [ :group | self correctionFor: group ].
	^ self mergeSecrets: secrets with: corrections! !

!VOMPDVSAuthority methodsFor: 'cryptography-mpdvs' stamp: 'NicolasVanderAuwera 5/6/2012 12:09'!
mergeSecrets: secrets with: corrections
	| verifiers group1 group2 witnessesSecrets final |
	final := OrderedCollection new.
	group1 := secrets first.
	group2 := secrets second.
	verifiers := Array with: group1 removeFirst with: group2 removeFirst.
	final := (corrections
		collectWithIndex: [ :correction :i | VOVerifierShare d: (verifiers at: i) correction: correction n: generator n ])
		asOrderedCollection.
	final
		addAll: (group1 collectWithIndex: [ :group :i | VOWitnessShare d1: (group1 at: i) d2: (group2 at: i) n: generator n ]).
	^ final! !

!VOMPDVSAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 14:45'!
numberOfParticipants
	^ CryptographyHelper defaultNumberOfWitnesses + 2! !

!VOMPDVSAuthority methodsFor: 'cryptography-mpdvs' stamp: 'NicolasVanderAuwera 5/9/2012 10:20'!
secretsFor: aGroup
	| share prod di invDen |
	^ (aGroup
		collect: [ :i | 
			share := self shares at: i.
			prod := (LagrangeCoefficient
				a: 0
				b: i
				interval: aGroup
				modulo: generator n) lc.
			di := share * prod.
			di isFraction
				ifTrue: [ 
					invDen := (ExtendedEuclidAlgorithm with: di denominator with: generator n) inverseModulo.
					di := di numerator * invDen ].
			di \\ generator n ]) asOrderedCollection! !

!VOMPDVSAuthority methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 3/28/2012 10:50'!
setup: bits
	^ generator := VORSAKeyPairGenerator bits: bits! !

!VOMPDVSAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/9/2012 10:12'!
shares
	shares
		ifNil: [ 
			| function |
			function := VOPolynome order: 1 constant: generator d max: generator n.
			shares := (1 to: self numberOfParticipants) collect: [ :i | function of: i mod: generator n ] ].
	^ shares! !

VOAuthority subclass: #VOWitnessAuthority
	instanceVariableNames: 'witnesses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOWitnessAuthority commentStamp: 'NicolasVanderAuwera 5/14/2012 12:45' prior: 0!
I have the set of witnesses for an election.

Instance Variables:
	witnesses	<(Matrix of: VOWitness) | (SequenceableCollection of: VOWitness)>!

!VOWitnessAuthority methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 14:40'!
givesShares: shares
	witnesses with: shares do: [ :witness :share | witness addShare: share ]! !

!VOWitnessAuthority methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 5/5/2012 14:44'!
initialize
	super initialize.
	witnesses := (1 to: CryptographyHelper defaultNumberOfWitnesses) collect: [ :i | VOWitness new ]! !

!VOWitnessAuthority methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/5/2012 14:43'!
signatureOn: message for: uid
	| signature |
	signature := message.
	witnesses do: [ :witness | signature := witness sign: signature for: uid ].
	^ signature! !

!VOWitnessAuthority methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/5/2012 14:43'!
simulationOn: message for: uid
	| simulation |
	simulation := message.
	witnesses do: [ :witness | simulation := witness simulation: simulation for: uid ].
	^ simulation! !

Object subclass: #VOPolynome
	instanceVariableNames: 'cst order coefficients max'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOPolynome commentStamp: 'NicolasVanderAuwera 5/14/2012 12:43' prior: 0!
I am a polynome. You can specify my order, the coefficients and the constant.

Instance Variables:
	cst	<AbstractSound | Collection | Color | DateAndTime | Duration | Number | Point | Timespan | TraitComposition | TraitDescription | TraitTransformation | VOBallot | VONode>
	order	<CharacterBlock | GRVersion | Magnitude | Point | SpDate | SpTimestamp | String | UUID | WAHttpVersion>
	coefficients	<Object>
	max	<ProtoObject | PseudoContext>!

!VOPolynome class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 1/13/2012 10:22'!
order: anInteger constant: anotherInteger coefficients: someCoefficients
	^ self new order: anInteger constant: anotherInteger coefficients: someCoefficients! !

!VOPolynome class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 3/6/2012 07:43'!
order: anInteger constant: anotherInteger max: maximum
	^ self new order: anInteger constant: anotherInteger max: maximum! !

!VOPolynome methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 2/29/2012 15:49'!
assertInvariant
	self assert: (order isInteger and: [ order > 0 ]).
	self assert: order = coefficients size! !

!VOPolynome methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/6/2012 08:06'!
coefficients
	^ coefficients! !

!VOPolynome methodsFor: 'generator' stamp: 'NicolasVanderAuwera 4/27/2012 11:05'!
generateCoefficients
	| random |
	random := SecureRandom new.
	^ (1 to: self order) collect: [ :i | random nextFrom: 1 to: max ]! !

!VOPolynome methodsFor: 'operations' stamp: 'NicolasVanderAuwera 3/28/2012 08:37'!
of: x
	| sum |
	sum := 0.
	1 to: order do: [ :k | sum := sum + ((coefficients at: k) * (x raisedToInteger: k)) ].
	^ cst + sum! !

!VOPolynome methodsFor: 'operations' stamp: 'NicolasVanderAuwera 3/6/2012 07:08'!
of: x mod: n
	^ (self of: x) \\ n! !

!VOPolynome methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/6/2012 07:07'!
order
	^ order! !

!VOPolynome methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 2/29/2012 15:49'!
order: anInteger constant: anotherInteger coefficients: someCoefficients
	order := anInteger.
	cst := anotherInteger.
	coefficients := someCoefficients.
	self assertInvariant ! !

!VOPolynome methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 5/6/2012 15:43'!
order: anInteger constant: anotherInteger max: maximum
	order := anInteger.
	cst := anotherInteger.
	max := maximum.
	coefficients := self generateCoefficients.
	self assertInvariant! !

Object subclass: #VOShare
	instanceVariableNames: 'uid n'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOShare commentStamp: 'NicolasVanderAuwera 5/14/2012 12:43' prior: 0!
I am generated by the MPDVS sub-agent.

Instance Variables:
	uid	<ChangeRecord | CharacterBlock | ECEntry | Election | FreeTypeFontFamilyMember | GRVersion | GoferResolvedReference | HelpTopic | MCDefinition | MCMockDependentItem | MCPatchOperation | Magnitude | OBClassReference | Point | SettingNode | SourceReference | SpDate | String | TextStyleAsFontFamilyMember | UUID | VOChoice | VOElectionResults | VOResult | VOShare | VOToken | VOUrn | WACallback | WAHttpVersion | WAPlugin | WAQualifiedValue | WAStatusItem>
	n	<ProtoObject | PseudoContext>!

!VOShare methodsFor: 'comparing' stamp: 'NicolasVanderAuwera 5/6/2012 11:37'!
<= aShare
	^ uid <= aShare uid! !

!VOShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/6/2012 12:02'!
n
	^ n! !

!VOShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/6/2012 12:01'!
n: anInteger
	n := anInteger! !

!VOShare methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/9/2012 10:07'!
sign: m
	self subclassResponsibility! !

!VOShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 11:08'!
uid
	^ uid! !

!VOShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 11:08'!
uid: anObject
	uid := anObject! !

VOShare subclass: #VOVerifierShare
	instanceVariableNames: 'd correction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOVerifierShare commentStamp: 'NicolasVanderAuwera 5/14/2012 12:43' prior: 0!
I am used by a verifier (mental booth or voter).

Instance Variables:
	d	<ProtoObject | PseudoContext>
	correction	<Collection | Color | Duration | Number | Point | VONode>!

!VOVerifierShare class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 5/6/2012 13:12'!
d: int1 correction: int2 n: int3
	^ self new
		d: int1;
		correction: int2;
		n: int3;
		yourself! !

!VOVerifierShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/28/2012 10:43'!
correction
	^ correction! !

!VOVerifierShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/22/2012 10:28'!
correction: anInteger
	correction := anInteger! !

!VOVerifierShare methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/7/2012 07:20'!
correctionFor: m
	^ (ExtendedEuclidAlgorithm with: (m raisedTo: correction * n modulo: n) with: n) inverseModulo! !

!VOVerifierShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/28/2012 10:43'!
d
	^ d! !

!VOVerifierShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/22/2012 10:28'!
d: anInteger
	d := anInteger! !

!VOVerifierShare methodsFor: 'process' stamp: 'NicolasVanderAuwera 3/28/2012 10:27'!
sign: m
	^ m raisedTo: d modulo: n! !

VOShare subclass: #VOWitnessShare
	instanceVariableNames: 'd1 d2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOWitnessShare commentStamp: 'NicolasVanderAuwera 5/14/2012 12:44' prior: 0!
I am used by a witness.

Instance Variables:
	d1	<ProtoObject | PseudoContext>
	d2	<ProtoObject | PseudoContext>!

!VOWitnessShare class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 3/22/2012 14:48'!
d1: int1 d2: int2 n: int3
	^ self new
		d1: int1;
		d2: int2;
		n: int3;
		yourself! !

!VOWitnessShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/28/2012 10:45'!
d1
	^ d1! !

!VOWitnessShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/22/2012 10:27'!
d1: anInteger
	d1 := anInteger! !

!VOWitnessShare methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 3/22/2012 10:27'!
d2: anInteger
	d2 := anInteger! !

!VOWitnessShare methodsFor: 'process' stamp: 'NicolasVanderAuwera 4/10/2012 10:49'!
sign: m
	self assert: [ m > 0 and: [ m < n ] ].
	^ m raisedTo: d1 modulo: n! !

!VOWitnessShare methodsFor: 'process' stamp: 'NicolasVanderAuwera 3/28/2012 10:27'!
simulation: m
	^ m raisedTo: d2 modulo: n! !

GRObject subclass: #VOWitness
	instanceVariableNames: 'shares signatures simulations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

VOWitness class
	instanceVariableNames: 'witnesses ports'!

!VOWitness commentStamp: 'NicolasVanderAuwera 5/14/2012 12:46' prior: 0!
I can sign a secret key or simulate a signature on a secret key, depending of the voter's choice.

Instance Variables:
	shares	<(Dictionary of: VOWitnessShare)>
	signatures	<Collection>
	simulations	<Collection>

Class Instance Variables:
	witnesses	<OrderedCollection>
	ports	<SequenceableCollection>!

VOWitness class
	instanceVariableNames: 'witnesses ports'!

!VOWitness class methodsFor: 'defaults' stamp: 'NicolasVanderAuwera 4/11/2012 09:30'!
defaultPort
	^ 1000! !

!VOWitness class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 4/11/2012 09:34'!
new: anInteger
	^ witnesses := (1 to: anInteger) collect: [ :i | (self new port: self untakenPort) start ]! !

!VOWitness class methodsFor: 'instance creation' stamp: 'NicolasVanderAuwera 4/11/2012 07:36'!
port: anInteger
	self assert: anInteger > 0.
	^ self new port: anInteger! !

!VOWitness class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 09:24'!
ports
	ports ifNil: [ ports := OrderedCollection new ].
	^ ports! !

!VOWitness class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 09:30'!
untakenPort
	| port |
	self ports isEmpty
		ifTrue: [ ports add: self defaultPort ]
		ifFalse: [ 
			port := ports last.
			ports add: port + 1 ].
	^ ports last! !

!VOWitness class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 09:35'!
witnesses
	witnesses ifNil: [ witnesses := OrderedCollection new ].
	^ witnesses! !

!VOWitness methodsFor: 'cryptography-mpdvs' stamp: 'NicolasVanderAuwera 4/11/2012 11:21'!
addShare: share
	shares at: share uid put: share! !

!VOWitness methodsFor: 'initialization' stamp: 'NicolasVanderAuwera 5/5/2012 13:36'!
initialize
	super initialize.
	shares := Dictionary new.
	signatures := OrderedCollection new.
	simulations := OrderedCollection new! !

!VOWitness methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/5/2012 13:36'!
sign: m for: anInteger
	signatures add: anInteger.
	^ (shares at: anInteger) sign: m! !

!VOWitness methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 13:36'!
signatures
	^ signatures! !

!VOWitness methodsFor: 'process' stamp: 'NicolasVanderAuwera 5/5/2012 13:36'!
simulation: m for: anInteger
	simulations add: anInteger.
	^ (shares at: anInteger) simulation: m! !

!VOWitness methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 13:37'!
simulations
	^ simulations! !

VOManager subclass: #VOMPDVSManager
	instanceVariableNames: 'numberOfWitnesses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOMPDVSManager commentStamp: 'NicolasVanderAuwera 5/14/2012 12:41' prior: 0!
I contain MPDVSs for each created election.

Instance Variables:
	numberOfWitnesses	<ProtoObject | PseudoContext>!

!VOMPDVSManager class methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 2/4/2012 16:44'!
organisationUnit
	^ 'MPDVS Authority'! !

!VOMPDVSManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 2/3/2012 08:05'!
port
	^ 9999! !

!VOMPDVSManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/19/2012 09:03'!
authorityClass
	^ VOMPDVSAuthority! !

!VOMPDVSManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:46'!
authorityFor: aProtocol
	^ VOMPDVSAuthority! !

!VOMPDVSManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/6/2012 11:32'!
handle: aSymbol
	super handle: aSymbol.
	aSymbol = #keygen
		ifTrue: [ self subAgentKeyGeneration ]! !

!VOMPDVSManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 4/11/2012 06:55'!
numberOfWitnesses
	^ numberOfWitnesses! !

!VOMPDVSManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 5/5/2012 14:26'!
numberOfWitnesses: anObject
	numberOfWitnesses := anObject! !

!VOMPDVSManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 8/26/2012 08:51'!
sendSecretsToWitnesses: secrets for: id
	| witnesses client |
	self configurations.
	witnesses := self configurations at: VOWitnessManager key.
	client := SocketStream openConnectionToHostNamed: witnesses host port: witnesses port.
	client
		sendCommand: #shares;
		sendCommand: id greaseString;
		sendObject: (secrets copyFrom: 3 to: secrets size).
	client close! !

!VOMPDVSManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/23/2012 07:48'!
subAgentKeyGeneration
	| subAgent secrets uid |
	subAgent := self authorityWithId: stream nextLine greaseInteger.
	uid := stream nextLine greaseInteger.
	subAgent setup: CryptographyHelper rsaKeyLength.
	secrets := subAgent keyGen.
	secrets do: [ :secret | secret uid: uid ].
	self sendSecretsToWitnesses: secrets for: subAgent id.
	stream sendObject: (secrets copyFrom: 1 to: 2)! !

VOManager subclass: #VOWitnessManager
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Volunti-MPDVS-Model'!

!VOWitnessManager commentStamp: 'NicolasVanderAuwera 5/14/2012 12:45' prior: 0!
I contains a VOWitnesses sub-agent, which has a set of witnesses, for each election.!

!VOWitnessManager class methodsFor: 'x509' stamp: 'NicolasVanderAuwera 5/5/2012 13:42'!
organisationUnit
	^ 'Witness Authority'! !

!VOWitnessManager class methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/5/2012 13:37'!
port
	^ 2795! !

!VOWitnessManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 8/19/2012 09:03'!
authorityClass
	^ VOWitnessAuthority! !

!VOWitnessManager methodsFor: 'accessing' stamp: 'NicolasVanderAuwera 7/23/2012 07:51'!
authorityFor: aProtocol
	^ VOWitnessAuthority! !

!VOWitnessManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 5/5/2012 14:22'!
handle: aSymbol
	super handle: aSymbol.
	aSymbol = #signature
		ifTrue: [ self signature ].
	aSymbol = #simulation
		ifTrue: [ self simulation ].
	aSymbol = #shares
		ifTrue: [ self receiveShares ]! !

!VOWitnessManager methodsFor: 'network' stamp: 'NicolasVanderAuwera 7/27/2012 08:00'!
receiveShares
	| subAgent id |
	id := self idOnStream.
	subAgent := self authorityWithId: id.
	subAgent givesShares: stream receiveObject! !

!VOWitnessManager methodsFor: 'process' stamp: 'NicolasVanderAuwera 7/23/2012 07:48'!
signature
	| uid m subAgent |
	subAgent := self authorityWithId: stream nextLine greaseInteger.
	uid := stream nextLine greaseInteger.
	m := stream nextLine greaseInteger.
	stream sendObject: (subAgent signatureOn: m for: uid)! !

!VOWitnessManager methodsFor: 'process' stamp: 'NicolasVanderAuwera 7/23/2012 07:48'!
simulation
	| uid m subAgent |
	subAgent := self authorityWithId: stream nextLine greaseInteger.
	uid := stream nextLine greaseInteger.
	m := stream nextLine greaseInteger.
	stream sendObject: (subAgent simulationOn: m for: uid)! !
VOWitnessAuthorityComponent initialize!
VOMPDVSComponent initialize!
